package audio;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import junit.framework.TestCase;
import main.Config;
import main.IntervalPlayer;
import utils.Console;
import utils.Utils;

public class Player implements Runnable
{
	private Config config;
	private ArrayList<Integer> fileOrder;
	private Mixer mixer;
	private boolean running;
	private IntervalPlayer main;
	private BufferedWriter logWriter;

	public Player(Config config, ArrayList<Integer> fileOrder, Mixer mixer, IntervalPlayer main, BufferedWriter logWriter)
	{
		this.config = config;
		this.fileOrder = fileOrder;
		this.mixer=mixer;
		this.main=main;
		this.logWriter=logWriter;
	}

	@Override
	public void run()
	{
		running=true;
		try {
			start();
		} catch (IOException | UnsupportedAudioFileException
				| LineUnavailableException | InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	public void stop(){running=false;}
	
	public void start() throws IOException, UnsupportedAudioFileException, LineUnavailableException, InterruptedException
	{
//		ArrayList<AudioInputStream> stream=new ArrayList<>();
		ArrayList<byte[]> data=new ArrayList<>(config.getAudioFile().size());
		System.out.println("Loading sounds.");
		AudioFormat format = null;
		{
			ByteArrayOutputStream out=new ByteArrayOutputStream();
			byte[] buf=new byte[256*256];
			for(Path p: config.getAudioFile())
			{
				byte[] b=Files.readAllBytes(p);
				AudioInputStream s=AudioSystem.getAudioInputStream(new ByteArrayInputStream(b));
//				stream.add(s);
//				while((len=s.read(buf))!=-1);
//				s.reset();
				int len;
				while((len=s.read(buf))!=-1) out.write(buf, 0, len);
				data.add(out.toByteArray());
				out.reset();
				format=s.getFormat();
			}
		}
		
		byte[] silent=new byte[(int)(format.getSampleRate()*(format.getSampleSizeInBits()/8)*format.getChannels()*config.getSilentMs()/1000)];
		
		DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
		SourceDataLine line = (SourceDataLine) mixer.getLine(info);
		line.open(format);
		line.start();
		
		for(Integer f: fileOrder)
		{
			if(!running) break;
//			AudioInputStream st=stream.get(f);
			
			System.out.println("Playing "+config.getAudioFile().get(f));
			logWriter.write(Utils.timeString()+"\t"+config.getAudioFile().get(f));
			logWriter.newLine();
			
//			while((len=st.read(buf))!=-1) line.write(buf, 0, len);
//			st.reset();
			byte[] d=data.get(f);
			int writeLen=0;
			while(writeLen<d.length) writeLen+=line.write(d, writeLen, d.length-writeLen);
			
			System.out.println("Silence "+config.getSilentMs()+" ms");
			logWriter.write(Utils.timeString()+"\t"+"Silence");
			logWriter.newLine();
			
//			Thread.sleep(config.getSilentMs());
			writeLen=0;
			while(writeLen<silent.length) writeLen+=line.write(silent, writeLen, silent.length-writeLen);
		}
		line.drain();
		line.close();
		
		System.out.println("End");
//		for(AudioInputStream st: stream) st.close();
		main.stopped();
	}
	
	public static class Test extends TestCase
	{
		public static void play() throws IOException, UnsupportedAudioFileException, LineUnavailableException, InterruptedException
		{
			Path dir=Paths.get("I:", "koumura", "IntervalPlayer", "Test");
			ArrayList<byte[]> byteList=new ArrayList<>();
//			for(int i=0; i<4; ++i)
//			{
//				Path file=dir.resolve("wave"+i+".wav");
//				byte[] wave0=Files.readAllBytes(file);
//				byteList.add(wave0);
//			}
			{
				Path file=dir.resolve("wave1.wav");
				byte[] wave0=Files.readAllBytes(file);
				byteList.add(wave0);
			}
			{
				Path file=dir.resolve("wave1.wav");
				byte[] wave0=Files.readAllBytes(file);
				byteList.add(wave0);
			}
			ArrayList<AudioInputStream> streamList=new ArrayList<>();
			for(byte[] b: byteList) streamList.add(AudioSystem.getAudioInputStream(new ByteArrayInputStream(b)));
			
			AudioFormat format = streamList.get(0).getFormat();
			AudioFormat format1 = streamList.get(1).getFormat();
			System.out.println(format.equals(format1));
			Console.psl(Utils.formatEquals(format, format1));
			
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
			
			Mixer.Info[] mixerInfo=AudioSystem.getMixerInfo();
			for(Mixer.Info mi: mixerInfo)
			{
				System.out.println(mi+" [Red]");
				Mixer m=AudioSystem.getMixer(mi);
				System.out.println(m.isLineSupported(info));
			}
			
			SourceDataLine line = (SourceDataLine) AudioSystem.getMixer(mixerInfo[2]).getLine(info);
//			SourceDataLine line = (SourceDataLine) AudioSystem.getLine(info);
			line.open(format);
			line.start();
			
			byte[] buf=new byte[256*16];
			int len;
			for(AudioInputStream stream: streamList)
			{
				while((len=stream.read(buf))!=-1) line.write(buf, 0, len);
				Thread.sleep(1000);
				stream.reset();
			}
			for(AudioInputStream stream: streamList)
			{
				while((len=stream.read(buf))!=-1) line.write(buf, 0, len);
				Thread.sleep(1000);
			}
			line.drain();
			line.close();
			
			for(AudioInputStream stream: streamList) stream.close();
		}
		
		public static void mixer() throws LineUnavailableException
		{
			AudioFormat format = new AudioFormat(44100, 16, 1, true, false); 
			DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
			Mixer.Info[] mixerInfo=AudioSystem.getMixerInfo();
			for(Mixer.Info mi: mixerInfo)
			{
				System.out.println(mi+" [Red]");
				Mixer m=AudioSystem.getMixer(mi);
//				{
//					Line.Info[] lineInfo=m.getSourceLineInfo();
//					for(Line.Info li: lineInfo)
//					{
//						System.out.println(li);
//						try
//						{
//							SourceDataLine line = (SourceDataLine) AudioSystem.getLine(li);
//							System.out.println(line);
//						}
//						catch(Exception e){e.printStackTrace();}
//					}
//				}
//				System.out.println();
//				{
//					Line.Info[] lineInfo=m.getTargetLineInfo();
//					for(Line.Info li: lineInfo)
//					{
//						System.out.println(li);
//						try
//						{
//							SourceDataLine line = (SourceDataLine) AudioSystem.getLine(li);
//							System.out.println(line);
//						}
//						catch(Exception e){e.printStackTrace();}
//					}
//				}
//				Line[] line=m.getTargetLines();
//				for(Line li: line)
//				{
//					System.out.println(li);
//				}
				System.out.println(m.isLineSupported(info));
			}
		}
		
		public static void in()
		{
			Line.Info[] info=AudioSystem.getSourceLineInfo(new Line.Info(SourceDataLine.class));
			for(Line.Info i: info) Console.pl(i);
		}
		
		public static void characterEnc() throws UnsupportedEncodingException
		{
			List<Mixer> mixer=Utils.supportedMixer();
			for(Mixer m: mixer)
			{
				String name=m.getMixerInfo().getName();
				Console.pl(name);
				name=new String(name.getBytes("Windows-31J"), "Windows-31J");
				Console.pl(name);
			}
//			Console.pl(convertToUnicode("-"));
			
		}
		/**
		 * Unicode文字列に変換する("あ" -> "\u3042")
		 * @param original
		 * @return
		 */
		private static String convertToUnicode(String original)
		{
		    StringBuilder sb = new StringBuilder();
		    for (int i = 0; i < original.length(); i++) {
		        sb.append(String.format("\\u%04X", Character.codePointAt(original, i)));
		    }
		    String unicode = sb.toString();
		    return unicode;
		}

		/**
		 * Unicode文字列から元の文字列に変換する ("\u3042" -> "あ")
		 * @param unicode
		 * @return
		 */
		private static String convertToOiginal(String unicode)
		{
		    String[] codeStrs = unicode.split("\\\\u");
		    int[] codePoints = new int[codeStrs.length - 1]; // 最初が空文字なのでそれを抜かす
		    for (int i = 0; i < codePoints.length; i++) {
		        codePoints[i] = Integer.parseInt(codeStrs[i + 1], 16);
		    }
		    String encodedText = new String(codePoints, 0, codePoints.length);
		    return encodedText;
		}

		public void os()
		{
			Console.pl(System.getProperty("os.name").toLowerCase());
		}
	}
}
