package main;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;

import utils.Console;

public class Config
{
	private long silentMs;
	private int numBlock;
	private ArrayList<Path> audioFile;
	private Path logDir;
	
	public Config(long silentMs, int numBlock, ArrayList<Path> audioFile, Path logDir)
	{
		this.silentMs = silentMs;
		this.numBlock = numBlock;
		this.audioFile = audioFile;
		this.logDir = logDir;
	}
	public Config()
	{
		audioFile=new ArrayList<>();
		logDir=Paths.get(".");
	}

	public long getSilentMs() {
		return silentMs;
	}

	public void setSilentMs(long silentMs) {
		this.silentMs = silentMs;
	}

	public ArrayList<Path> getAudioFile() {
		return audioFile;
	}

	public void setAudioFile(ArrayList<Path> audioFile) {
		this.audioFile = audioFile;
	}

	public Path getLogDir() {
		return logDir;
	}

	public void setLogDir(Path logDir) {
		this.logDir = logDir;
	}

	public int getNumBlock() {
		return numBlock;
	}

	public void setNumBlock(int numBlock) {
		this.numBlock = numBlock;
	}
	
	public void write(Path path) throws IOException
	{
		BufferedWriter writer=Files.newBufferedWriter(path);
		writer.write(String.valueOf(silentMs));
		writer.newLine();
		writer.write(String.valueOf(numBlock));
		writer.newLine();
		writer.write(logDir.toString());
		writer.newLine();
		writer.write(String.valueOf(audioFile.size()));
		writer.newLine();
		for(Path p: audioFile)
		{
			writer.write(p.toString());
			writer.newLine();
		}
		writer.close();
	}
	
	public static Config read(Path path) throws IOException
	{
		BufferedReader reader=Files.newBufferedReader(path);
		long silentMs=Long.parseLong(reader.readLine());
		int numBlock=Integer.parseInt(reader.readLine());
		Path logDir=new File(reader.readLine()).toPath();
		int audioFileSize=Integer.parseInt(reader.readLine());
		ArrayList<Path> audioFile=new ArrayList<>(audioFileSize);
		for(int i=0; i<audioFileSize; ++i) audioFile.add(new File(reader.readLine()).toPath());
		return new Config(silentMs, numBlock, audioFile, logDir);
	}
}
