package main;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file. 
 */

import java.awt.BorderLayout;
import java.awt.Font;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.Mixer;
import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.commons.math3.random.MersenneTwister;

import audio.Player;
import swing.MainFrame;
import utils.Utils;

public class IntervalPlayer
{
	private MainFrame mainFrame;
	private Config config;
	private List<Mixer> mixer;
	private MersenneTwister random;
	private Thread playThread;
	private Player player;
	private BufferedWriter logWriter;
	private static final Path CONFIG_PATH=Paths.get("./config");
	
	public IntervalPlayer(Config config)
	{
		this.config=config;
		mainFrame=new MainFrame(this);
		mainFrame.getPanel().setConfigValue(config);
		updateMixer(null);
	}
	
	public void updateMixer(AudioFormat format)
	{
		if(format==null&&mainFrame.getPanel().getSoundFileLine().getFirst().getBrowseButton().getFormat()!=null)
			format=mainFrame.getPanel().getSoundFileLine().getFirst().getBrowseButton().getFormat();
		if(format==null) mixer=Utils.supportedMixer();
		else mixer=Utils.supportedMixer(format);
		mainFrame.getPanel().setMixer(mixer);
	}
	
	public void run()
	{
		mainFrame.setVisible(true);
	}
	public MainFrame getMainFrame() {
		return mainFrame;
	}

	public Config getConfig() {
		return config;
	}

	public void setConfig(Config config) {
		this.config = config;
	}

	public Mixer mixer(int index){return mixer.get(index);}
	
	public MersenneTwister random()
	{
		if(random==null) random=new MersenneTwister(System.currentTimeMillis());
		return random;
	}
	
	public void play(Config config, ArrayList<Integer> fileOrder, Mixer mixer, Path logPath) throws IOException
	{
		logWriter=Files.newBufferedWriter(logPath);
		player=new Player(config, fileOrder, mixer, this, logWriter);
		playThread=new Thread(player);
		playThread.start();
	}
	
	public void stopped() throws IOException
	{
		logWriter.close();
		mainFrame.getPanel().stopped();
	}
	
	public void stop() throws InterruptedException, IOException
	{
		player.stop();
		playThread.join();
		stopped();
	}
	
	public static void main(String ...arg)
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e)
		{
			e.printStackTrace();
		}
		
		File fileConfig=CONFIG_PATH.toFile();
		Config config=null;
		if(fileConfig.exists())
		{
			try
			{
				config=Config.read(CONFIG_PATH);
			}
			catch(IOException e){e.printStackTrace();}
		}
		if(config==null) config=new Config();
		IntervalPlayer main=new IntervalPlayer(config);
		main.mainFrame.getPanel().add(new JLabel("<html>&copy; 2015 Takuya KOUMURA</html>"), "span,align center");
		main.getMainFrame().getPanel().getOutputText().start();
		System.out.println("--------------- Interval Player ---------------");
		System.out.println("Developed by Takuya KOUMURA");
		System.out.println("For further information, please visit our web site:");
		System.out.println("http://cycentum.miraiserver.com/intervalplayer/");
		System.out.println("--------------------------------------------");
		System.out.println();
		main.run();
	}
	
	public static void writeConfig(Config config) throws IOException
	{
		config.write(CONFIG_PATH);
	}
}
