package standardoutput;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

public abstract class StandardOutputDisplayer
{
	private Stream  outStream, errStream;
	private PrintStream outOrig, errOrig;

	public abstract void addOut(String string) throws Exception;
	public abstract void addErr(String string) throws Exception;
	
	public void start()
	{
		if(outStream==null)
		{
			outOrig=System.out;
			errOrig=System.err;
			outStream=new Stream(this);
			errStream=new Stream(this);
			System.setOut(new PrintStream(outStream));
			System.setErr(new PrintStream(errStream));
		}
	}

	public void end()
	{
		if(outStream!=null)
		{
			System.setOut(outOrig);
			System.setErr(errOrig);
			outStream=null;
		}
	}
	
	public void flush()
	{
		try
		{
			addOut(outStream.toString());
			addErr(errStream.toString());
		}
		catch (Exception e){e.printStackTrace();}
		outStream.reset();
		errStream.reset();
	}
	
	public static class Stream extends ByteArrayOutputStream
	{
		private StandardOutputDisplayer displayer;

		public Stream (StandardOutputDisplayer displayer)
		{
			this.displayer=displayer;
		}

		@Override
		public synchronized void write(byte[] b, int off, int len)
		{
			super.write(b, off, len);
			displayer.flush();
		}

		@Override
		public synchronized void write(int b)
		{
			super.write(b);
			displayer.flush();
		}

		@Override
		public void write(byte[] b) throws IOException
		{
			super.write(b);
			displayer.flush();
		}
	}
}
