package swing;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.nio.file.Path;

import javax.swing.JFrame;

import main.Config;
import main.IntervalPlayer;
import utils.Console;
import utils.Utils;

@SuppressWarnings("serial")
public class MainFrame extends JFrame
{
	private IntervalPlayer main;
	private MainPanel panel;

	public MainFrame(IntervalPlayer main)
	{
		this.main=main;
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowList(this));
		panel=new MainPanel(this);
		add(panel);
		setPreferredSize(Dimensions.MAIN_FRAME);
		setMinimumSize(Dimensions.MAIN_FRAME);
		Utils.icon(this);
	}
	
	public IntervalPlayer getMain() {
		return main;
	}

	public void setMain(IntervalPlayer main) {
		this.main = main;
	}

	public MainPanel getPanel() {
		return panel;
	}

	public void setPanel(MainPanel panel) {
		this.panel = panel;
	}

	private static class WindowList extends WindowAdapter
	{
		private MainFrame frame;
		
		private WindowList(MainFrame frame) {
			this.frame = frame;
		}

		@Override
		public void windowClosing(WindowEvent event)
		{
			Config config=frame.getMain().getConfig();
			frame.getPanel().getConfigValue(config);
			try
			{
				IntervalPlayer.writeConfig(config);
			}
			catch(IOException e){e.printStackTrace();}
			System.exit(0);
		}
	}
}
