package swing;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.nio.file.Path;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import utils.Utils;

public class SoundFileLine
{
	private MainPanel mainPanel;
	private JLabel pathLabel;
	private BrowseButton browseButton;
	private JButton removeButton;
	private ChangeList changeListener;
	
	public SoundFileLine(MainPanel mainPanel, Path prevDirectory, JPanel parentPanel)
	{
		this.mainPanel = mainPanel;
		
		pathLabel=new JLabel();
		pathLabel.setMaximumSize(Dimensions.PATH_LABEL_MAX);
		pathLabel.setPreferredSize(Dimensions.PATH_LABEL_MAX);
		parentPanel.add(pathLabel);
		
		changeListener=new ChangeList(this);
		browseButton=new BrowseButton(prevDirectory, pathLabel, false, "*.wav, *.wave", new String[]{"wav", "wave"}, changeListener);
		parentPanel.add(browseButton);
		
		removeButton=new JButton("Remove");
		parentPanel.add(removeButton, "wrap");
		removeButton.addActionListener(new RemoveList(this));
	}
	
	public BrowseButton getBrowseButton() {
		return browseButton;
	}
	
	public JLabel getPathLabel() {
		return pathLabel;
	}

	public JButton getRemoveButton() {
		return removeButton;
	}

	public void setRemoveButton(JButton removeButton) {
		this.removeButton = removeButton;
	}

	public void setPathLabel(JLabel pathLabel) {
		this.pathLabel = pathLabel;
	}

	public void setBrowseButton(BrowseButton browseButton) {
		this.browseButton = browseButton;
	}

	public void setToolTipText()
	{
		if(browseButton.getSelected()==null) return;
		String text="";
		try
		{
			text=Utils.formatString(browseButton.getSelected());
		}
		catch (UnsupportedAudioFileException | IOException e){e.printStackTrace();}
		pathLabel.setToolTipText(text);
	}
	
	private static class ChangeList implements BrowseButton.ChangeListener
	{
		private SoundFileLine line;
		
		private ChangeList(SoundFileLine line) {
			this.line = line;
		}

		@Override
		public void changed(BrowseButton button)
		{
			if(button==line.mainPanel.getSoundFileLine().getLast().browseButton)
			{
				line.mainPanel.addSoundFileLine();
			}
			line.setToolTipText();
			line.mainPanel.checkNewSoundFile(line);
		}
	}
	
	private static class RemoveList implements ActionListener
	{
		private SoundFileLine line;
		
		private RemoveList(SoundFileLine line) {
			this.line = line;
		}
		
		@Override
		public void actionPerformed(ActionEvent e)
		{
			if(line.browseButton.getSelected()==null) return;
			line.mainPanel.removeSoundFileLine(line);
		}
	}
}
