package utils;

/*
Copyright (C) 2015 Takuya KOUMURA
http://cycentum.miraiserver.com/intervalplayer/

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Attribution notices are included in the NOTICE file.
 */

import java.awt.Component;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

import javax.imageio.ImageIO;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;


public class Utils
{
//	private static final SimpleDateFormat TIME_FORMAT=new SimpleDateFormat("HH:mm:ss.SSS");
	private static final SimpleDateFormat TIME_FORMAT=new SimpleDateFormat("HH:mm:ss");
	private static BufferedImage ICON=null;
	
	public static String formatString(Path path) throws UnsupportedAudioFileException, IOException
	{
		AudioFormat format=AudioSystem.getAudioFileFormat(path.toFile()).getFormat();
		StringBuffer buf=new StringBuffer();
		buf.append("<html><table border=\"1\" cellspacing=\"0\">");
		buf.append("<tr><td>Path</td><td>"+path.toString()+"</td></tr>");
		buf.append("<tr><td>Sampling rate</td><td>"+format.getSampleRate()+"</td></tr>");
		buf.append("<tr><td>Bit depth</td><td>"+format.getSampleSizeInBits()+"</td></tr>");
		buf.append("<tr><td>Channels</td><td>"+format.getChannels()+"</td></tr>");
		buf.append("</table></html>");
		return buf.toString();
	}
	
	public static boolean formatEquals(AudioFormat format0, AudioFormat format1)
	{
		return format0.getSampleRate()==format1.getSampleRate() &&
				format0.getEncoding()==format1.getEncoding() &&
				format0.getChannels()==format1.getChannels() &&
				format0.getSampleSizeInBits()==format1.getSampleSizeInBits();
	}
	
	public static List<Mixer> supportedMixer(AudioFormat format)
	{
		DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
		
		Mixer.Info[] mixerInfo=AudioSystem.getMixerInfo();
		List<Mixer> mixer=Arrays.asList(mixerInfo).stream().map(i->{
			Mixer m=AudioSystem.getMixer(i);
			if(!m.isLineSupported(info)) return null;
			return m;
			}).filter(m->m!=null).collect(Collectors.toList());
		return mixer;
	}
	
	public static List<Mixer> supportedMixer()
	{
		Line.Info[] infos=AudioSystem.getSourceLineInfo(new Line.Info(SourceDataLine.class));
		List<Mixer> mixer=new LinkedList<>();
		for(Line.Info info: infos)
		{
			Mixer.Info[] mixerInfo=AudioSystem.getMixerInfo();
			mixer.addAll(Arrays.asList(mixerInfo).stream().map(i->{
				Mixer m=AudioSystem.getMixer(i);
				if(!m.isLineSupported(info)) return null;
				return m;
				}).filter(m->m!=null).collect(Collectors.toList()));
		}
		mixer=mixer.stream().distinct().collect(Collectors.toList());
		return mixer;
	}
	
	public static void errorMessage(Component parent, String message)
	{
		JOptionPane.showMessageDialog(parent, message, "Error", JOptionPane.ERROR_MESSAGE);
	}
	
	public static boolean isSupportedFormat(Mixer mixer, AudioFormat format)
	{
		DataLine.Info info = new DataLine.Info(SourceDataLine.class, format);
		return mixer.isLineSupported(info);
	}
	
	public static String timeString()
	{
		return TIME_FORMAT.format(new Date(System.currentTimeMillis()));
	}
	
	public static void icon(JFrame frame)
	{
		if(ICON==null)
		{
			try
			{
//				ICON=ImageIO.read(Paths.get("I:", "koumura", "IntervalPlayer", "icon.png").toFile());
				ICON=ImageIO.read(Paths.get("./resources/icon.png").toFile());
			}
			catch(IOException e){e.printStackTrace();}
		}
		frame.setIconImage(ICON);
	}
}
